package ru.ulstu.is.sbapp.socialNetwork.controller;

import org.springframework.web.bind.annotation.*;
import ru.ulstu.is.sbapp.configuration.WebConfiguration;
import ru.ulstu.is.sbapp.socialNetwork.dto.MyUserDTO;
import ru.ulstu.is.sbapp.socialNetwork.models.UserModel;
import ru.ulstu.is.sbapp.socialNetwork.services.MyUserService;


import javax.validation.Valid;
import java.util.List;


@RestController
@RequestMapping(WebConfiguration.REST_API + "/users")
public class MyUserController {
    private final MyUserService userService;

    public MyUserController(MyUserService userService){
        this.userService = userService;
    }

    @GetMapping("/{id}")
    public MyUserDTO getUser(@PathVariable Long id) {
        return new MyUserDTO(userService.findUser(id));
    }

    @GetMapping("")
    public List<MyUserDTO> getUsers() {
        return userService.findAllUsers().stream().map(MyUserDTO::new).toList();
    }

    @PostMapping("")
    public MyUserDTO createUser(@RequestBody @Valid MyUserDTO user) {
        UserModel result = userService.addUser(user.getName(), user.getCity());
        userService.updateCommunities(result.getId(), user.getCommunity());
        return new MyUserDTO(userService.updateMusics(result.getId(), user.getMusic()));
    }

    @PatchMapping("/{id}")
    public MyUserDTO updateUser(@PathVariable Long id,
                                @RequestBody @Valid MyUserDTO user) {
        UserModel result = userService.updateUser(id, user.getName());
        userService.updateCommunities(result.getId(), user.getCommunity());
        return new MyUserDTO(userService.updateMusics(result.getId(), user.getMusic()));
    }

    @PatchMapping("/add_music/{id}")
    public MyUserDTO addMusic(@PathVariable Long id, @RequestParam Long music_id) {
        return new MyUserDTO(userService.addMusic(id, music_id));
    }

    @PatchMapping("/add_community/{id}")
    public MyUserDTO addCommunity(@PathVariable Long id, @RequestParam Long community_id) {
        return new MyUserDTO(userService.addCommunity(id, community_id));
    }
    @GetMapping("/search")
    public List<MyUserDTO> searchUsers(@RequestParam("city") String city) {
        List<UserModel> users = userService.findUserByCity(city);
        return users.stream().map(MyUserDTO::new).toList();
    }

    @DeleteMapping("/{id}")
    public MyUserDTO deleteUser(@PathVariable Long id) {
        return new MyUserDTO(userService.deleteUser(id));
    }

}
