package ru.ulstu.is.sbapp;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import ru.ulstu.is.sbapp.socialNetwork.models.Community;
import ru.ulstu.is.sbapp.socialNetwork.models.UserModel;
import ru.ulstu.is.sbapp.socialNetwork.services.CommunityService;
import ru.ulstu.is.sbapp.socialNetwork.services.MyUserService;

import javax.persistence.EntityNotFoundException;
import java.util.List;

@SpringBootTest
public class JpaCommunityTest {
    private static final Logger log = LoggerFactory.getLogger(JpaUserTest.class);
    @Autowired
    MyUserService userService;

    @Autowired
    CommunityService communityService;

    @Test
    void testUserCreate() {
        communityService.deleteAllCommunities();
        final Community community = communityService.addCommunity("CodeGPT");
        log.info(community.toString());
        Assertions.assertNotNull(community.getId());
    }

    @Test
    void testUserReadNotFound() {
        communityService.deleteAllCommunities();
        Assertions.assertThrows(EntityNotFoundException.class, () -> communityService.findCommunity(-1L));
    }

    @Test
    void findUser() {
        communityService.deleteAllCommunities();
        final Community community = communityService.addCommunity("CodeGPT");
        log.info(community.toString());
        final Community findCommunity = communityService.findCommunity(community.getId());
        log.info(findCommunity.toString());
        Assertions.assertEquals(community.toString(), findCommunity.toString());
    }

    @Test
    void testReadAll() {
        communityService.deleteAllCommunities();
        userService.deleteAllUsers();
        final Community community = communityService.addCommunity("CodeGPT");
        log.info(community.toString());
        final List<Community> communities = communityService.findAllCommunities();
        Assertions.assertEquals(communities.size(), 1);
    }

    @Test
    void updateCommunity() {
        userService.deleteAllUsers();
        communityService.deleteAllCommunities();
        final Community community = communityService.addCommunity("CodeGPT 2");
        communityService.updateCommunity(community.getId(), "CodeGPT 3");
        final Community findCommunity = communityService.findCommunity(community.getId());
        log.info(findCommunity.toString());
        Assertions.assertEquals(findCommunity.getName(), "CodeGPT 3");
    }

    @Test
    void deleteCommunity() {
        userService.deleteAllUsers();
        communityService.deleteAllCommunities();
        final Community community = communityService.addCommunity("CodeGPT");
        log.info(community.toString());
        communityService.deleteCommunity(community.getId());
        Assertions.assertThrows(EntityNotFoundException.class, () -> communityService.findCommunity(community.getId()));
    }


    @Test
    void addUser() {
        userService.deleteAllUsers();
        communityService.deleteAllCommunities();
        userService.findAllUsers();
        final Community community = communityService.addCommunity("CodeGPT");
        final UserModel user = userService.addUser("its_mailov", "Moscow");
        userService.addCommunity(user.getId(), community.getId());
        final Community findCommunity = communityService.findCommunity(community.getId());
        Assertions.assertEquals(findCommunity.getUsers().get(0).getId(), user.getId());
    }
}

